<?php

namespace App\Http\Controllers\User;

use App\Constants\Status;
use App\Http\Controllers\Controller;
use App\Models\AdminNotification;
use App\Rules\FileTypeValidate;
use Illuminate\Http\Request;
use App\Models\Category;
use App\Models\City;
use App\Models\Listing;
use App\Models\ListingImage;
use App\Models\Region;
use Illuminate\Validation\ValidationException;

class ListingController extends Controller {
    public function myListings() {
        $pageTitle  = 'My Listings';
        $cities     = City::active()->get();

        $listings = Listing::with('category', 'city')->where('user_id', auth()->user()->id)->searchable(['title'])->filter(['city_id', 'status'])->orderBy('id', 'desc')->paginate(getPaginate());

        return view($this->activeTemplate . 'user.listing.all', compact('pageTitle', 'cities', 'listings'));
    }

    public function form() {
        $pageTitle  = 'Create Listing';
        $cities  = City::active()->get();
        $categories = Category::active()->get();
        $regions = Region::orderBy('name', 'asc')->get();
        $activePlan = auth()->user()->activePlan;
        return view($this->activeTemplate . 'user.listing.form', compact('pageTitle', 'cities', 'categories', 'activePlan', 'regions'));
    }

    public function edit($id) {
        $pageTitle  = 'Edit listing';
        $cities     = City::active()->get();
        $categories = Category::active()->get();
        $regions = Region::orderBy('name', 'asc')->get();
        $listing   = Listing::where('user_id', auth()->user()->id)->with('images')->findOrFail($id);

        $images = [];
        foreach ($listing->images as $key => $image) {
            $img['id']  = $image->id;
            $img['src'] = getImage(getFilePath('listing') . '/' . $image->name);
            $images[]   = $img;
        }
        $activePlan = auth()->user()->activePlan;
        return view($this->activeTemplate . 'user.listing.form', compact('pageTitle','regions', 'cities', 'categories', 'listing', 'images', 'activePlan'));
    }

    public function store(Request $request, $id = 0) {
        $this->validation($request, $id);

        if ($id) {
            $listing = Listing::findOrFail($id);
            $notification = 'Listing updated successfully';
            $adminNotificationTitle = 'Listing approval request';
        } else {
            $listing = new listing();
            $notification = 'Listing created successfully';
            $adminNotificationTitle = 'New listing approval request';
        }

        if ($request->hasFile('display_image')) {
            try {
                $old = $listing->display_image;
               // $listing->display_image = fileUploader($request->display_image, getFilePath('listing'), getFileSize('listing'), $old);
                //$listing->display_image = fileUploader($request->display_image, storage_path('app/public/'.'assets/images/listing'), getFileSize('listing'), $old);
                $listing->display_image = fileUploader($request->display_image, storage_path('app/public/'.'assets/images/listing'), null, $old);
            } catch (\Exception $exp) {
                $notify[] = ['error', 'Couldn\'t upload your listing image'];
                return back()->withNotify($notify);
            }
        }

        $user = auth()->user();

        $listing->user_id          = $user->id;
        $listing->listing_id      = getTrx(8);
        $listing->title            = $request->title;
        $listing->description      = $request->description;
        $listing->category_id      = $request->category_id;
        $listing->city_id          = $request->city_id;
        $listing->location         = $request->location;
        $listing->open_time        = $request->open_time;
        $listing->close_time       = $request->close_time;
        $listing->day_off          = $request->day_off ?? [];
        $listing->amenities        = $request->amenities ?? [];
        $listing->tags             = $request->tags ?? [];
        $listing->videos           = $request->videos ?? [];
        $listing->status           = Status::LISTING_PENDING;
        $listing->map              = $request->map;
        $listing->save();
        $this->insertImages($request, $listing, $id);

        $adminNotification = new AdminNotification();
        $adminNotification->user_id = $user->id;
        $adminNotification->title = $adminNotificationTitle;
        $adminNotification->click_url = route('admin.listing.details', $listing->id);
        $adminNotification->save();

        $notify[] = ['success', $notification];
        return back()->withNotify($notify);
    }

    protected function validation($request, $id) {

        $imageRules  = $id ? 'nullable' : 'required';
        $user        = auth()->user();
        $plan        = $user->activePlan;

        if (!$plan) {
            throw ValidationException::withMessages(['error' => 'Your current plan has expired!']);
        }

        $request->validate([
            'title'              => 'required|string|max:255',
            'description'        => 'required|string',
            'category_id'        => 'required|exists:categories,id',
            'city_id'            => 'required|exists:cities,id',
            'location'           => 'required|string|max:255',
            'tags'               => 'nullable|array|min:1|max:10',
            'tags.*'             => 'required|string',
            'amenities'          => ['nullable', 'array', 'max:' . $plan->amenities],
            'amenities.*.name'   => 'required|string',
            'amenities.*.value'  => 'required|string',
            'day_off'            => 'nullable|array|min:1|max:7',
            'day_off.*'          => 'string|in:saturday,sunday,monday,tuesday,wednesday,thursday,friday',
            'open_time'          => 'nullable|date_format:H:i',
            'close_time'         => 'nullable|required_with:open_time|date_format:H:i',
            'display_image'      => [$imageRules, 'image', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
            'images'             => ['nullable', 'array', 'max:' . $plan->images],
            'images*'            => ['nullable', 'images', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
            'videos'             => ['nullable', 'array', 'max:' . $plan->videos],
            'videos.*'           => 'required|url',
            'map'                => 'nullable|url',
        ], [
            'amenities.max'      => "Please follow maximum limit of amenities",
            'amenities.*.name'   => "Amenity name fields must not be empty",
            'amenities.*.value'  => "Amenity value fields must not be empty",
        ]);

        if (!$id) {

            $listingsCount = Listing::approved()->where('user_id', $user->id)->count();

            if ($plan->listings >= $listingsCount) {
                $notify[] = ['error', 'The maximum amount of listing has already been added'];
                return to_route('pricing')->withNotify($notify);
            }
        }


        $oldImages = $request->old ?? [];
        $newImages = $request->images ?? [];
        $galleryImages = $oldImages + $newImages;


        if ($plan->images < count($galleryImages)) {
            throw ValidationException::withMessages(['error' => 'The maximum amount of image has already been uploaded']);
        }


        if (!$plan->allow_map && $request->map) {
            throw ValidationException::withMessages(['error' => 'You are not allowed to add map']);
        }
    }

    protected function insertImages($request, $listing, $id) {
        //$path = getFilePath('listing');
        $path = storage_path('app/public/'.'assets/images/listing');
        if ($id) {
            $this->removeImages($request, $listing, $path);
        }

        $hasImages = $request->file('images');
        if ($hasImages) {
            $size = getFileSize('listing');
            $images = [];

            if (!file_exists($path)) {
                mkdir($path, 0755, true);
            }

            foreach ($hasImages as $file) {
                try {
                    //$name = fileUploader($file, $path, $size);
                    $name = fileUploader($file, $path);
                    $image        = new ListingImage();
                    $image->listing_id = $listing->id;
                    $image->name = $name;
                    $images[]     = $image;
                } catch (\Exception $exp) {
                    $notify[] = ['error', 'Couldn\'t upload listing images'];
                    return back()->withNotify($notify);
                }
            }
            $listing->images()->saveMany($images);
        }
    }

    protected function removeImages($request, $listing, $path) {
        $previousImages = $listing->images->pluck('id')->toArray();
        $imageToRemove  = array_values(array_diff($previousImages, $request->old ?? []));
        foreach ($imageToRemove as $item) {
            $listingImage   = ListingImage::find($item);
            @unlink($path . '/' . $listingImage->name);
            $listingImage->delete();
        }
    }

    public function switchStatus($id, $status) {
        $user     = auth()->user();
        $listing = Listing::where('user_id', $user->id)->where('id', $id)->firstOrFail();

        if ($status == Status::LISTING_DISABLED) {
            $notification = "Listing disabled successfully";
        } elseif ($status == Status::LISTING_APPROVED) {
            $status       = Status::LISTING_PENDING;
            $notification = "Your approval request send successfully. Please wait for admin approval.";

            $adminNotification = new AdminNotification();
            $adminNotification->user_id = $user->id;
            $adminNotification->title = 'Listing approval request';
            $adminNotification->click_url = route('admin.listing.details', $listing->id);
            $adminNotification->save();
        } else {
            abort(403);
        }

        $listing->status = $status;
        $listing->save();
        $notify[] = ['success',  $notification];
        return back()->withNotify($notify);
    }
}
