<?php

namespace App\Http\Controllers\User;

use App\Constants\Status;
use App\Http\Controllers\Controller;
use App\Lib\GoogleAuthenticator;
use App\Models\Category;
use App\Models\City;
use App\Models\FavoriteListing;
use App\Models\GatewayCurrency;
use App\Models\Listing;
use App\Models\Plan;
use App\Models\Review;
use Illuminate\Http\Request;

class UserController extends Controller {

    public function home() {
        $pageTitle = 'Dashboard';
        $user      = auth()->user();
        $plan      = $user->activePlan;
        $data['total_listing']    = Listing::whereUserId($user->id)->count();
        $data['approved_listing'] = Listing::whereUserId($user->id)->approved()->count();
        $data['pending_listing']  = Listing::whereUserId($user->id)->pending()->count();
        $data['disabled_listing'] = Listing::whereUserId($user->id)->disabled()->count();
        $data['rejected_listing'] = Listing::whereUserId($user->id)->rejected()->count();
        $data['featured_listing'] = Listing::whereUserId($user->id)->approved()->featured()->count();
        $listings = Listing::with('category', 'city')->where('user_id', auth()->user()->id)->take(10)->get();
        return view($this->activeTemplate . 'user.dashboard', compact('pageTitle', 'user', 'data', 'listings', 'plan'));
    }

    public function depositHistory(Request $request) {
        $pageTitle = 'Payment History';
        $deposits = auth()->user()->deposits()->searchable(['trx'])->with(['gateway'])->orderBy('id', 'desc')->paginate(getPaginate());
        return view($this->activeTemplate . 'user.deposit_history', compact('pageTitle', 'deposits'));
    }

    public function attachmentDownload($fileHash) {
        $filePath = decrypt($fileHash);
        $extension = pathinfo($filePath, PATHINFO_EXTENSION);
        $general = gs();
        $title = slug($general->site_name) . '- attachments.' . $extension;
        $mimetype = mime_content_type($filePath);
        header('Content-Disposition: attachment; filename="' . $title);
        header("Content-Type: " . $mimetype);
        return readfile($filePath);
    }

    public function userData() {
        $user = auth()->user();
        if ($user->profile_complete == 1) {
            return to_route('user.home');
        }
        $pageTitle = 'Complete Your Profile';
        return view($this->activeTemplate . 'user.user_data', compact('pageTitle', 'user'));
    }

    public function userDataSubmit(Request $request) {
        $user = auth()->user();
        if ($user->profile_complete == Status::YES) {
            return to_route('user.home');
        }
        $request->validate([
            'firstname' => 'required',
            'lastname' => 'required',
        ]);
        $user->firstname = $request->firstname;
        $user->lastname = $request->lastname;
        $user->address = [
            'country' => @$user->address->country,
            'address' => $request->address,
            'state' => $request->state,
            'zip' => $request->zip,
            'city' => $request->city,
        ];
        $user->profile_complete = Status::YES;
        $user->save();

        $notify[] = ['success', 'Registration process completed successfully'];
        return to_route('user.home')->withNotify($notify);
    }

    function choosePlan(Request $request, $id) {
        $request->validate([
            'price_type' => 'required|in:monthly,yearly'
        ]);

        $plan = Plan::active()->findOrFail($id);
        $pageTitle = "Buy Plan";
        $type = $request->price_type;

        $gatewayCurrency = GatewayCurrency::whereHas('method', function ($gate) {
            $gate->where('status', Status::ENABLE);
        })->with('method')->orderby('method_code')->get();

        return view($this->activeTemplate . 'user.buy_plan', compact('pageTitle', 'type', 'gatewayCurrency', 'plan'));
    }

    public function storeRating(Request $request, $id) {
        $request->validate([
            'review'      => 'required|string',
            'rating'      => 'required|integer',
        ]);

        $listing = Listing::findOrFail($id);

        if ($listing->user_id == auth()->user()->id) {
            $notify[] = ['error', 'Sorry, You can\'t give review on your own listing!'];
            return back()->withNotify($notify);
        }

        $existingReview = Review::where('user_id', auth()->id())->where('listing_id', $listing->id)->first();

        if ($existingReview) {
            $notify[] = ['error', 'You\'ve already reviewed this listing'];
            return back()->withNotify($notify);
        }

        $review = new Review();
        $review->user_id     = auth()->id();
        $review->listing_id  = $listing->id;
        $review->review      = $request->review;
        $review->rating      = $request->rating;
        $review->save();

        $notify[] = ['success', 'Review submitted successfully'];
        return back()->withNotify($notify);
    }


    public function addToFavorite($id) {

        $listing = Listing::find($id);

        if (!$listing) {
            return response()->json(['error' => 'Something is wrong']);
        }

        $favoriteExcite = FavoriteListing::where('user_id', auth()->user()->id)->where('listing_id', $id)->first();

        if ($favoriteExcite) {
            $favoriteExcite->delete();
            return response()->json(['success' => 'Removed from favorite list']);
        }

        $favorite = new FavoriteListing();
        $favorite->listing_id = $id;
        $favorite->user_id = auth()->id();
        $favorite->save();
        return response()->json(['success' => 'Added to favorite list']);
    }

    public function favoriteListings() {
        $pageTitle  = "Favorite listings";

        $listings = Listing::whereHas('favoriteListing', function ($favoriteListing) {
            $favoriteListing->where('user_id', auth()->id());
        })->WhereFavoriteList()
            ->withCount('reviews')
            ->whereHas('user.activePlan')
            ->with('category', 'city', 'user.activePlan')
            ->searchable(['tags', 'location', 'title'])
            ->filter(['category_id', 'city_id']);

        $listings = $listings->orderBy('id', 'DESC')->paginate(getPaginate());


        $categories = Category::active()->get();
        $cities     = City::active()->get();
        return view($this->activeTemplate . 'listings', compact('pageTitle', 'listings', 'categories', 'cities'));
    }
}
